package org.apache.beehive.controls.runtime.generator;
/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Collection;

import com.sun.mirror.declaration.ClassDeclaration;
import com.sun.mirror.declaration.MethodDeclaration;
import com.sun.mirror.declaration.ParameterDeclaration;
import com.sun.mirror.type.ClassType;
import com.sun.mirror.type.PrimitiveType;
import com.sun.mirror.type.ReferenceType;
import com.sun.mirror.type.TypeMirror;

import org.apache.beehive.controls.api.packaging.FeatureInfo;


/**
 * The AptMethodSet method represents a collection of AptMethod objects.  It contains special
 * support for method overloading, to ensure that overloaded method objects contained within
 * the set will each have a unique index value.
 *
 * @see org.apache.beehive.controls.runtime.generator.AptMethod#setIndex
 */ 
public class AptMethodSet<T extends AptMethod>
{
    /**
     * Adds a new method to the list.  Also detects overloaded methods and ensures that they
     * will receive a unique index value.
     */
    public void add(T method)
    {
        // Add to the list of managed methods
        _methods.add(method);

        // Ensure that all added methods have a unique index
        Object nameValue = _nameMap.get(method.getName());
        if (nameValue == null)
        {
            // first method with this name, considered unique (for now)
            _nameMap.put(method.getName(), method);
        }
        else
        {
            int nextIndex;
            if (nameValue instanceof AptMethod)
            {
                // 2nd method with this name, add index to original and start indexing
                ((AptMethod)nameValue).setIndex(0);
                nextIndex = 1;
            }
            else
            {
                // 3rd (or later) method with this name, continue indexing
                nextIndex = ((Integer)nameValue).intValue();
            }

            method.setIndex(nextIndex++);
            _nameMap.put(method.getName(), nextIndex);
        }
    }

    public Collection<T> getMethods()
    {
        return _methods;
    }

    public int size()
    {
        return _methods.size();
    }
        
    HashMap _nameMap = new HashMap();   // method name -> a single (unique) AptMethod or next index
    ArrayList<T> _methods = new ArrayList<T>();
}
